

drawing_multiple = false;
multi_depth = 0;
debug_indent = "";


function table.shallowcopy(table)
    copy = {}
    for k, v in pairs(table) do
        copy[k] = v
    end
    return copy
end


function ResetTable(before, now, name)
    if name == nil then
        name = "table"
    end
    if #now ~= #before then
        local diff = #before - #now;

        if diff >= 0 then
            for j = 1, diff do
                local pos = diff - j + 1;
                local action = before[pos]

                action.uses_remaining = action.uses_remaining - 1
                ActionUsesRemainingChanged( action.inventoryitem_id, action.uses_remaining )
                table.insert(now, 1, action)
            end
        else
            for j = #now, diff + #now + 1, -1 do
                local pos = j;
                table.remove(now, pos)
            end
        end
    end
end


function CheckIsShotValid(action_mana_required, action)
    if action_mana_required > mana then
        OnNotEnoughManaForAction()
        table.insert( discarded, action )
        return false
    end

    if action.uses_remaining == 0 then
        table.insert( discarded, action )
        return false
    end

    return true;
end


-- Don't do anything foolish when we shouldn't do it
local original_order_deck = order_deck
function order_deck(...)
    if not drawing_multiple then
        return original_order_deck(...)
    end
end


-- Don't do anything foolish when we shouldn't do it
local original_move_discarded_to_deck = move_discarded_to_deck
function move_discarded_to_deck(...)
    if not drawing_multiple then
        return original_move_discarded_to_deck(...)
    end
end


local original_draw_actions = draw_actions
function draw_actions(how_many, instant_reload_if_empty)
    local val = "None"
    if #deck > 0 then
        val = deck[1].id
    end

    if drawing_multiple then
        local original_deck = table.shallowcopy(deck);
        local original_hand = table.shallowcopy(hand);
        local original_discards = table.shallowcopy(discarded);

        local retval = original_draw_actions(how_many, instant_reload_if_empty)

        -- Reset hands when running duplicate action
        ResetTable(original_hand, hand, "hand")
        ResetTable(original_discards, discarded, "discarded")
        ResetTable(original_deck, deck, "deck")

        return retval
    else
        return original_draw_actions(how_many, instant_reload_if_empty)
    end
end


function multi_draw_action( how_many, instant_reload_if_empty )
    if reflecting then  return  end

	local action = nil
    local was_drawing_multi = drawing_multiple;
    local depth_at_first = multi_depth;
    local original_deck = table.shallowcopy(deck);
    local original_hand = table.shallowcopy(hand);
    local original_discards = table.shallowcopy(discarded);

    if not drawing_multiple then
        state_cards_drawn = state_cards_drawn + 1
    else
        multi_depth = multi_depth + 1;
    end

    local card_state_memory = state_cards_drawn;

    if ( #deck > 0 ) then
		-- draw from current multi depth
        action = deck[1]

        for i = 1, how_many do
            local last_iteration = false;
            if (i == how_many) then
                last_iteration = true;
            end
            if action ~= nil then
                if action.mana == nil then
                    action_mana_required = ACTION_MANA_DRAIN_DEFAULT
                end

                -- update mana
                local action_mana_required = action.mana
                if not CheckIsShotValid(action_mana_required, action) then
                    if not was_drawing_multi then
                        table.remove(deck, 1);
                    end
                    return false;
                end

                mana = mana - action_mana_required
                if not was_drawing_multi then
                    if not last_iteration then
                        drawing_multiple = true;
                    else
                        drawing_multiple = false;
                    end

                    table.remove(deck, 1)
                    play_action( action )
                    drawing_multiple = false;

                    if not last_iteration then
                        ResetTable(original_deck, deck, "deck")
                        ResetTable(original_hand, hand, "hand")
                        ResetTable(original_discards, discarded, "discarded")
                    end
                else
                    table.remove(deck, 1)
                    play_action( action )

                    ResetTable(original_deck, deck, "deck")
                    ResetTable(original_hand, hand, "hand")
                    ResetTable(original_discards, discarded, "discarded")
                    state_cards_drawn = card_state_memory;
                end

                if ( #deck <= 0 ) then
                    if multi_depth == 0 then
                        ResetTable(original_deck, deck, "deck")
                        ResetTable(original_hand, hand, "hand")
                        ResetTable(original_discards, discarded, "discarded")
                    end

                    if instant_reload_if_empty then
                        move_discarded_to_deck()
                        order_deck()
                        start_reload = true
                    else
                        reloading = true
                        return true
                    end
                end

                -- Reset basic data
                multi_depth = depth_at_first;
            end
        end
    end

	return true
end
